using System;
using System.Collections.Generic;
using System.Text;

namespace Be.Windows.Forms
{
    /// <summary>
    /// Specifies the case of hex characters in the HexBox control
    /// </summary>
    public enum HexCasing
    {
        /// <summary>
        /// Converts all characters to uppercase.
        /// </summary>
        Upper = 0,
        /// <summary>
        /// Converts all characters to lowercase.
        /// </summary>
        Lower = 1
    }
}
