//------------------------------------------------------------------------------
// <auto-generated>
//     This code was generated by a tool.
//     Runtime Version:4.0.30319.17929
//
//     Changes to this file may cause incorrect behavior and will be lost if
//     the code is regenerated.
// </auto-generated>
//------------------------------------------------------------------------------

namespace Be.HexEditor {
    using System;
    
    
    /// <summary>
    ///   A strongly-typed resource class, for looking up localized strings, etc.
    /// </summary>
    // This class was auto-generated by the StronglyTypedResourceBuilder
    // class via a tool like ResGen or Visual Studio.
    // To add or remove a member, edit your .ResX file then rerun ResGen
    // with the /str option, or rebuild your VS project.
    [global::System.CodeDom.Compiler.GeneratedCodeAttribute("System.Resources.Tools.StronglyTypedResourceBuilder", "4.0.0.0")]
    [global::System.Diagnostics.DebuggerNonUserCodeAttribute()]
    [global::System.Runtime.CompilerServices.CompilerGeneratedAttribute()]
    internal class strings {
        
        private static global::System.Resources.ResourceManager resourceMan;
        
        private static global::System.Globalization.CultureInfo resourceCulture;
        
        [global::System.Diagnostics.CodeAnalysis.SuppressMessageAttribute("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode")]
        internal strings() {
        }
        
        /// <summary>
        ///   Returns the cached ResourceManager instance used by this class.
        /// </summary>
        [global::System.ComponentModel.EditorBrowsableAttribute(global::System.ComponentModel.EditorBrowsableState.Advanced)]
        internal static global::System.Resources.ResourceManager ResourceManager {
            get {
                if (object.ReferenceEquals(resourceMan, null)) {
                    global::System.Resources.ResourceManager temp = new global::System.Resources.ResourceManager("Be.HexEditor.strings", typeof(strings).Assembly);
                    resourceMan = temp;
                }
                return resourceMan;
            }
        }
        
        /// <summary>
        ///   Overrides the current thread's CurrentUICulture property for all
        ///   resource lookups using this strongly typed resource class.
        /// </summary>
        [global::System.ComponentModel.EditorBrowsableAttribute(global::System.ComponentModel.EditorBrowsableState.Advanced)]
        internal static global::System.Globalization.CultureInfo Culture {
            get {
                return resourceCulture;
            }
            set {
                resourceCulture = value;
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Byte {0}.
        /// </summary>
        internal static string Byte {
            get {
                return ResourceManager.GetString("Byte", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to English.
        /// </summary>
        internal static string English {
            get {
                return ResourceManager.GetString("English", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The file does not exist!.
        /// </summary>
        internal static string FileDoesNotExist {
            get {
                return ResourceManager.GetString("FileDoesNotExist", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Find reached end of file!.
        /// </summary>
        internal static string FindOperationEndOfFile {
            get {
                return ResourceManager.GetString("FindOperationEndOfFile", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to German.
        /// </summary>
        internal static string German {
            get {
                return ResourceManager.GetString("German", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Italian.
        /// </summary>
        internal static string Italian {
            get {
                return ResourceManager.GetString("Italian", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Japanese.
        /// </summary>
        internal static string Japanese {
            get {
                return ResourceManager.GetString("Japanese", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to No byte is selected.
        /// </summary>
        internal static string NoByteSelected {
            get {
                return ResourceManager.GetString("NoByteSelected", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to File is locked by another process and cannot be opened!.
        /// </summary>
        internal static string OpenFailed {
            get {
                return ResourceManager.GetString("OpenFailed", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to The file is locked. Do you want to open it in read-only mode?.
        /// </summary>
        internal static string OpenReadonly {
            get {
                return ResourceManager.GetString("OpenReadonly", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Program restart required for settings to take affect!.
        /// </summary>
        internal static string ProgramRestartSettings {
            get {
                return ResourceManager.GetString("ProgramRestartSettings", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to  [read-only].
        /// </summary>
        internal static string Readonly {
            get {
                return ResourceManager.GetString("Readonly", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Russian.
        /// </summary>
        internal static string Russian {
            get {
                return ResourceManager.GetString("Russian", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Do you want to save changes?.
        /// </summary>
        internal static string SaveChangesQuestion {
            get {
                return ResourceManager.GetString("SaveChangesQuestion", resourceCulture);
            }
        }
        
        /// <summary>
        ///   Looks up a localized string similar to Simplified Chinese.
        /// </summary>
        internal static string SimplifiedChinese {
            get {
                return ResourceManager.GetString("SimplifiedChinese", resourceCulture);
            }
        }
    }
}
